// Social profiles (users, groups, etc.)

// TODO: rename
.info {
  display: block;
}

ul.latest-visitors {
  margin: 0;
  padding: 0;

  li {
    list-style: none;

    img.avatar {
      margin: 2px;
      border-radius: 20%;
      width: 20px;
      height: 20px;
    }
  }
}
